/**
 * Copyright (c) 2000-2013 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.portal.kernel.util;

import com.liferay.portal.kernel.exception.PortalException;

import java.net.URI;
import java.net.URISyntaxException;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Akos Thurzo
 */
public class CSSUtil {

	public static final String[] PATH_REPLACEMENTS = new String[] {
		"@portal_ctx@", "@theme_image_path@"
	};

	public static String transformCss(String content, String originalPath)
		throws PortalException {

		return transformCss(content, originalPath, StringPool.BLANK);
	}

	public static String transformCss(
			String content, String originalPath, String newPath)
		throws PortalException {

		originalPath = originalPath.replace(
			CharPool.BACK_SLASH, CharPool.SLASH);

		Matcher matcher = _CSS_URL_PATTERN.matcher(content);

		StringBundler modifiedContentSB = new StringBundler();

		int pos = 0;

		URI newUri = null;

		try {
			if (Validator.isNotNull(newPath)) {
				newPath = newPath.replace(CharPool.BACK_SLASH, CharPool.SLASH);

				newUri = new URI(newPath);
			}
		}
		catch (URISyntaxException urise) {
			throw new PortalException(urise);
		}

		while (matcher.find()) {
			modifiedContentSB.append(content.substring(pos, matcher.start()));

			String url = matcher.group(2);

			if (StringUtil.contains(url, PATH_REPLACEMENTS, true)) {
				modifiedContentSB.append(matcher.group());

				pos = matcher.end();

				continue;
			}

			URI uri = null;

			try {
				if (url.startsWith(StringPool.SLASH)) {
					uri = new URI(url);
				}
				else {
					uri = new URI(
						originalPath.concat(StringPool.SLASH).concat(url));
				}
			}
			catch (URISyntaxException urise) {
				throw new PortalException(urise);
			}

			uri = uri.normalize();

			if (newUri != null) {
				uri = newUri.relativize(uri);
			}

			modifiedContentSB.append(matcher.group(1));
			modifiedContentSB.append(uri.toString());
			modifiedContentSB.append(matcher.group(3));

			pos = matcher.end();
		}

		modifiedContentSB.append(content.substring(pos, content.length()));

		return modifiedContentSB.toString();
	}

	private static final Pattern _CSS_URL_PATTERN = Pattern.compile(
		"(?<!@import )(url\\((?!\\s*\\))['\"]?)([^'\"]+?)(['\"]?\\))");

}