<%--
/**
 * Copyright (c) 2000-2013 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */
--%>

<%@ include file="/html/portlet/journal/init.jsp" %>

<%
String referringPortletResource = ParamUtil.getString(request, "referringPortletResource");

JournalArticle article = (JournalArticle)request.getAttribute(WebKeys.JOURNAL_ARTICLE);

long classNameId = BeanParamUtil.getLong(article, request, "classNameId");

String structureId = BeanParamUtil.getString(article, request, "structureId");

String deleteButtonLabel = "delete-this-version";

if ((article != null) && article.isDraft()) {
	deleteButtonLabel = "discard-draft";
}
%>

<div class="article-toolbar" id="<portlet:namespace />articleToolbar"></div>

<aui:script use="aui-toolbar,aui-dialog-iframe,liferay-util-window">
	var permissionPopUp = null;

	var toolbarChildren = [];

	var form = A.one('#<portlet:namespace />fm1');

	var formChanged = false;

	var hasUnsavedChanges = function() {
		var unsavedChanges = formChanged;

		if (!unsavedChanges && typeof CKEDITOR !== 'undefined') {
			unsavedChanges = CKEDITOR.instances.<portlet:namespace />articleContent.checkDirty();
		}

		return unsavedChanges;
	};

	form.delegate(
		'change',
		function(event) {
			formChanged = true;
		},
		':input'
	);

	<c:if test="<%= Validator.isNotNull(structureId) && (classNameId == JournalArticleConstants.CLASSNAME_ID_DEFAULT) %>">
		toolbarChildren.push(
			{
				icon: 'preview',
				label: '<%= UnicodeLanguageUtil.get(pageContext, "preview") %>',
				on: {
					click: function(event) {
						event.domEvent.preventDefault();

						if (!hasUnsavedChanges()) {
							window.<portlet:namespace />journalPortlet.previewArticle();
						}
						else {
							if (confirm('<%= UnicodeLanguageUtil.get(pageContext, "this-article-has-changes-and-to-preview-the-content-you-have-to-save-the-article-as-draft") %>')) {
								window.<portlet:namespace />journalPortlet.saveArticle('<%= Constants.PREVIEW %>');
							}
						}
					}
				}
			}
		);
	</c:if>

	<c:if test="<%= Validator.isNotNull(structureId) %>">
		toolbarChildren.push(
			{
				icon: 'download',
				id: '<portlet:namespace />downloadArticleContentButton',
				label: '<%= UnicodeLanguageUtil.get(pageContext, "download") %>'
			}
		);
	</c:if>

	<c:if test="<%= JournalArticlePermission.contains(permissionChecker, article, ActionKeys.PERMISSIONS) %>">
		<liferay-security:permissionsURL windowState="<%= LiferayWindowState.POP_UP.toString() %>"
			modelResource="<%= JournalArticle.class.getName() %>"
			modelResourceDescription="<%= article.getTitle(locale) %>"
			resourceGroupId="<%= String.valueOf(article.getGroupId()) %>"
			resourcePrimKey="<%= String.valueOf(article.getResourcePrimKey()) %>"
			var="permissionsURL"
		/>

		toolbarChildren.push(
			{
				handler: function(event) {
					if (!permissionPopUp) {
						permissionPopUp = Liferay.Util.openWindow(
							{
								dialog: {
									align: Liferay.Util.Window.ALIGN_CENTER,
									cssClass: 'portlet-asset-categories-admin-dialog permissions-change',
									width: 700
								},
								id: '<portlet:namespace />articlePermissions',
								title: '<%= UnicodeLanguageUtil.get(pageContext, "permissions") %>',
								uri: '<%= permissionsURL %>'
							}
						);
					}
					else {
						permissionPopUp.iframe.node.get('contentWindow.location').reload(true);
					}

					permissionPopUp.show();
					permissionPopUp.centered();

				},
				icon: 'permissions',
				label: '<%= UnicodeLanguageUtil.get(pageContext, "permissions") %>'
			}
		);
	</c:if>

	<c:if test="<%= !article.isExpired() && JournalArticlePermission.contains(permissionChecker, article, ActionKeys.EXPIRE) && !article.isApproved() %>">
		toolbarChildren.push(
			{
				handler: function() {
					<portlet:namespace />expireArticle();
				},
				icon: 'expire',
				label: '<%= UnicodeLanguageUtil.get(pageContext, "expire-this-version") %>'
			}
		);
	</c:if>

	<c:if test="<%= JournalArticlePermission.contains(permissionChecker, article, ActionKeys.DELETE) && !article.isApproved() && !article.isDraft() %>">
		toolbarChildren.push(
			{
				handler: function() {
					<portlet:namespace />deleteArticle();
				},
				icon: 'delete',
				label: '<liferay-ui:message key="<%= deleteButtonLabel %>" />'
			}
		);
	</c:if>

	<portlet:renderURL var="viewHistoryURL">
		<portlet:param name="struts_action" value="/journal/view_article_history" />
		<portlet:param name="redirect" value="<%= currentURL %>" />
		<portlet:param name="referringPortletResource" value="<%= referringPortletResource %>" />
		<portlet:param name="groupId" value="<%= String.valueOf(article.getGroupId()) %>" />
		<portlet:param name="articleId" value="<%= article.getArticleId() %>" />
	</portlet:renderURL>

	toolbarChildren.push(
		{
			handler: function(event) {
				window.location = '<%= viewHistoryURL %>';
			},
			icon: 'history',
			label: '<%= UnicodeLanguageUtil.get(pageContext, "view-history") %>'
		}
	);

	new A.Toolbar(
		{
			activeState: false,
			boundingBox: '#<portlet:namespace />articleToolbar',
			children: toolbarChildren
		}
	).render();
</aui:script>
